function [dvar] = derivative(var)
% var = [q(1) q(2) q(3) q(4) q(5) p(1) p(2) p(3)]
%     = [th   ph   alp  xp   yp   th1  ph1  alp1]
% Returns derivative of var as obtained from the Lagrange's Equation

global LastBeta LastTheta
global dq dp

q = [var(1) var(2) var(3) var(4) var(5)];
p = [var(6) var(7) var(8)];

tm = p45(q, p, LastBeta, LastTheta);
p(4) = tm(1);
p(5) = tm(2);

for n = 1:5,
    dpv(n, n) = dp;
    dqv(n, n) = dq;
end

p45p1 = (p45(q, p + dpv(1,:), LastBeta, LastTheta) - tm) / dp;
p4p1 = p45p1(1); % dp(4) / dp(1)
p5p1 = p45p1(2); % dp(5) / dp(1)
p45p2 = (p45(q, p + dpv(2,:), LastBeta, LastTheta) - tm) / dp;
p4p2 = p45p2(1); % dp(4) / dp(2)
p5p2 = p45p2(2); % dp(5) / dp(2)
for k = 1:3,
    p45qk = (p45(q + dqv(k,:), p, LastBeta, LastTheta) - tm) / dq;
    p4q(k) = p45qk(1); % dp(4) / dq(k)
    p5q(k) = p45qk(2); % dp(5) / dq(k)
end


Lag = I_L(q, p, LastBeta, LastTheta);

% (d^2)L / (dp(j) * dp(n))
for n = 1:5,
    tmpL = I_L(q, p + dpv(n,:), LastBeta, LastTheta);
    for j = 1:5,
        Lpp(n, j) = (I_L(q, p + dpv(n, :) + dpv(j, :), LastBeta, LastTheta) - tmpL - I_L(q, p + dpv(j, :), LastBeta, LastTheta) + Lag) / (dp*dp);
    end
end

% Lpq(n, j) = (d^2)L / (dp(j) * dq(n))
for n = 1:5,
    tmpL = I_L(q + dqv(n,:), p, LastBeta, LastTheta);
    for j = 1:5,
        Lpq(n, j) = (I_L(q + dqv(n, :), p + dpv(j, :), LastBeta, LastTheta) - tmpL - I_L(q, p + dpv(j, :), LastBeta, LastTheta) + Lag) / (dp*dq);
    end
end

% Lq(j) = dL / dq(j)
for j = 1:5,
    Lq(j) = (I_L(q + dqv(j, :), p, LastBeta, LastTheta) - Lag) / dq;
end

A = I_A(q, LastBeta, LastTheta);


for j = 1:3,
    E(j) = Lq(j) + A(j, 1)*Lq(4) + A(j, 2)*Lq(5);
    for n = 1:5,
        E(j) = E(j) - Lpq(n, j)*p(n) - A(j, 1)*Lpq(n, 4)*p(n) - A(j, 2)*Lpq(n, 5)*p(n);
        D(n, j) = Lpp(n, j) + A(j, 1)*Lpp(n, 4) + A(j, 2)*Lpp(n, 5);
    end
end

F4 = 0;
F5 = 0;
for k = 1:3,
    F4 = F4 + p4q(k)*p(k);
    F5 = F5 + p5q(k)*p(k);
end

for j = 1:3,
    G(j, 1) = D(1, j) + D(4, j)*p4p1 + D(5, j)*p5p1;
    G(j, 2) = D(2, j) + D(4, j)*p4p2 + D(5, j)*p5p2;
    G(j, 3) = D(3, j);
    H(j) = E(j) - F4*D(4, j) - F5*D(5, j);
end

%G,
%rank(G),
%H,

tmpdp = inv(G) * H';
%tmpdp,

LastBeta = bta(var(1), var(2));
LastTheta = var(1);

for n = 1:5,
    dvart(n) = p(n);
end
for n = 1:3,
    dvart(5 + n) = tmpdp(n);
end

dvar = dvart;
